unit ULsCmdItem;

interface

uses
 SysUtils;

type
  TLsCmdItem = class(TObject)
    function IsValidLine: Boolean;
    function IsDirectory: Boolean;
    function IsLink: Boolean;
    function GetName: String;
    function GetPermissionsStr: String;
    function GetOwnerStr: String;
    function GetGroupStr: String;
    function GetDateStr: String;
    function GetLsStr: String;
    function GetFileSize: Integer;
    function GetINodeBlocks: Integer;
    procedure Parse;
  private
    mLine: String;
    posFNameStart: Integer;
    parsedStr: array [0..10] of String;
    bParsed: Boolean;
    nbrOfItems: Integer;
  public
    constructor Create(const lsLine: String; fPos: Integer);
    destructor Destroy; override;
  end;

implementation

constructor TLsCmdItem.Create(const lsLine: String; fPos: Integer);
begin
  posFNameStart:=fPos;
  mLine:=lsLine;
  inherited Create();  // Initialize inherited parts
end;

destructor TLsCmdItem.Destroy;
begin
  inherited Destroy;
end;

function TLsCmdItem.IsValidLine: Boolean;
begin
 IsValidLine:=True;
end;

function TLsCmdItem.IsDirectory: Boolean;
begin
 IsDirectory:=(Length(mLine)>0) and (UpCase(mLine[1]) = 'D');
end;

function TLsCmdItem.IsLink: Boolean;
begin
 IsLink:=(Length(mLine)>0) and (UpCase(mLine[1]) = 'L');
end;

function TLsCmdItem.GetINodeBlocks: Integer;
begin
 GetINodeBlocks:=-1;
 Parse;
// New in Version 0.9 - Handling the exception
 if bParsed then
  try
    GetINodeBlocks:=StrToInt(parsedStr[1]);
  except
    on EConvertError do GetINodeBlocks:=-1;
  end;
end;

function TLsCmdItem.GetFileSize: Integer;
begin
 GetFileSize:=-1;
 Parse;
// New in Version 0.9 - Handling the exception
 if bParsed then
  try
    GetFileSize:=StrToInt(parsedStr[4]);
  except
    on EConvertError do GetFileSize:=-1;
  end;
end;

function TLsCmdItem.GetPermissionsStr: String;
begin
 GetPermissionsStr:='';
 Parse;
 if bParsed then
   GetPermissionsStr:=parsedStr[0];
end;

function TLsCmdItem.GetOwnerStr: String;
begin
 GetOwnerStr:='';
 Parse;
 if bParsed then
   GetOwnerStr:=parsedStr[2];
end;

function TLsCmdItem.GetGroupStr: String;
begin
 GetGroupStr:='';
 Parse;
 if bParsed then
   GetGroupStr:=parsedStr[3];
end;

function TLsCmdItem.GetDateStr: String;
begin
 GetDateStr:='';
 Parse;
 if bParsed then
   GetDateStr:=parsedStr[5] +' ' +parsedStr[6] +' ' +parsedStr[7];
end;

procedure TLsCmdItem.Parse;
var
 i: Integer;
 curIsSpaceChar: Boolean;
 curDelimitPosStart: Integer;
begin
 if (posFNameStart < 0) and not bParsed then
  begin
   nbrOfItems:=0;
   curIsSpaceChar:=False;
   curDelimitPosStart:=1;
   for i := 1 to Length(mLine) do
    begin
     if (mLine[i] = ' ') and (nbrOfItems <= 7) then
      begin
       if not curIsSpaceChar then
        begin
         curIsSpaceChar:=True;
         parsedStr[nbrOfItems]:=Copy(mLine,curDelimitPosStart,i-curDelimitPosStart);
         Inc(nbrOfItems);
        end;
      end
     else
      begin
       if curIsSpaceChar then
        begin
         curDelimitPosStart:=i;
         curIsSpaceChar:=False;
        end;
      end;
    end;
   if (nbrOfItems > 7) then
    begin
     bParsed:=True;
     parsedStr[8]:=Copy(mLine,curDelimitPosStart,Length(mLine));
    end;
  end;
end;

function TLsCmdItem.GetName: String;
var
 retStr: String;
begin
 Parse;
 retStr:='';
 if (posFNameStart >= 0) then
   retStr:=Copy(mLine,posFNameStart,Length(mLine))
 else if bParsed then
   retStr:=parsedStr[8];
 GetName:=retStr;
end;

function TLsCmdItem.GetLsStr: String;
var
 retStr: String;
 linkPos: Integer;
begin
 if IsLink then
  begin
   linkPos:=Pos(' -> ',mLine);
   if (linkPos <> 0) then
     retStr:=Copy(mLine,linkPos+4,Length(mLine))
   else
     retStr:=GetName;
  end
 else
   retStr:=GetName;
 GetLsStr:=retStr;
end;

end.
