unit USetup;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Registry, ComCtrls, contnrs, Menus, Spin,
  ShellAPI, UHostEntry, ULsCmdItem, ULoginForm, ExtCtrls, ImgList;

const
  defaultFilePos = 56;
  defaultTimeout = 15;
  WM_STD_OUT_CHAR     = WM_APP + 400;
  WM_STD_ERR_CHAR     =	WM_APP + 401;
  WM_STATS_CHAR	      =	WM_APP + 402;
  WM_STATS_SIZE       =	WM_APP + 403;
  WM_STATS_PERCENT    =	WM_APP + 404;
  WM_STATS_ELAPSED    =	WM_APP + 405;
  WM_RET_ERR_CNT      = WM_APP + 406;
  WM_LS_RET_ERR_CNT   = WM_APP + 407;

type
  TSetupForm = class(TForm)
    NameEdit: TEdit;
    UsernameEdit: TEdit;
    UsernameCheckBox: TCheckBox;
    DefaultPasswordCheckBox: TCheckBox;
    HostLabel: TLabel;
    HostEdit: TEdit;
    OKButton: TButton;
    Label1: TLabel;
    NameLabel: TLabel;
    ButtonCancel: TButton;
    OptionsGroupBox: TGroupBox;
    CheckBoxPreserveAttributes: TCheckBox;
    PortEdit: TEdit;
    Label2: TLabel;
    ButtonFilePosDefault: TButton;
    SpinEditFilePos: TSpinEdit;
    Label4: TLabel;
    SpinEditTimeout: TSpinEdit;
    ListViewTest: TListView;
    ButtonTest: TButton;
    ImageListDefault: TImageList;
    ImageListDefaultBig: TImageList;
    NoResponseTimer: TTimer;
    CheckBoxHideConsole: TCheckBox;
    CheckBoxFixPos: TCheckBox;
    ButtonExamples: TButton;
    LocalDirEdit: TEdit;
    LocDirLabel: TLabel;
    procedure OKButtonClick(Sender: TObject);
    procedure ButtonCancelClick(Sender: TObject);
    procedure ButtonFilePosDefaultClick(Sender: TObject);
    procedure ButtonTestClick(Sender: TObject);
    procedure SetPassword(const pwd: String);
    procedure SpinEditFilePosChange(Sender: TObject);
    procedure NoResponseTimerTimer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure CheckBoxFixPosClick(Sender: TObject);
    procedure ButtonExamplesClick(Sender: TObject);
  protected
    procedure WMStdOutCharReceived(var Message: TMessage);     message WM_STD_OUT_CHAR;
    procedure WMStdErrCharReceived(var Message: TMessage);     message WM_STD_ERR_CHAR;
    procedure WMLsCmdErrCntReceived(var Message: TMessage);    message WM_LS_RET_ERR_CNT;
  private
    testing: Boolean;
    pwdOk: Boolean;
    noResponseCnt: Integer;
    lineStr, pwdStr: String;
  public
    { Public declarations }
  end;

implementation

{$R *.DFM}

procedure TSetupForm.WMStdOutCharReceived(var Message: TMessage);
begin
 WMStdErrCharReceived(Message);
end;

procedure TSetupForm.WMStdErrCharReceived(var Message: TMessage);
var
 ch: char;
 ListItem: TListItem;
 lsItem: TLsCmdItem;
 nameStr: String;
begin
 ch := Chr(Message.WParam);
 if (ch <> #10) then
   lineStr:=lineStr+ch
 else
  begin
// New in Version 0.9
   lsItem:=TLsCmdItem.Create(lineStr,-1);
   with lsItem do
    if IsValidLine() then
     begin
      nameStr:=GetName;
      if (Length(nameStr) > 0) and (nameStr <> '.') and (nameStr <> '..') then
       begin
        ListItem := ListViewTest.Items.Add;
        ListItem.Caption := GetLsStr;
        if IsDirectory() then
          ListItem.ImageIndex:=1
        else if lsItem.IsLink() then
          ListItem.ImageIndex:=4
        else
          ListItem.ImageIndex:=3;
       end;
     end;
   lsItem.Free;
   lineStr:='';
  end;
end;

procedure TSetupForm.WMLsCmdErrCntReceived(var Message: TMessage);
begin
 Screen.Cursor := crDefault;
 testing:=False;
 noResponseCnt:=-1;
end;

procedure TSetupForm.SetPassword(const pwd: String);
begin
 pwdOk:=True;
 pwdStr:=pwd;
end;

procedure TSetupForm.OKButtonClick(Sender: TObject);
begin
 if not testing then
  begin
   ModalResult:=mrOK;
   // Replace backslash with slash -just to be nice
   HostEdit.Text:=StringReplace(HostEdit.Text,'\','/',[rfReplaceAll]);
  end;
end;

procedure TSetupForm.ButtonCancelClick(Sender: TObject);
begin
 if not testing then
   ModalResult:=mrCancel;
end;

procedure TSetupForm.ButtonFilePosDefaultClick(Sender: TObject);
begin
 SpinEditFilePos.Value:=defaultFilePos;
 ButtonTestClick(Sender);
end;

procedure TSetupForm.ButtonTestClick(Sender: TObject);
var
 consoleMode: Integer;
 loginForm: TLoginForm;
 lsCmdStr: String;
 tempCmd,tempParam: array[0..10000] of char;
begin
 if testing then
   Exit;
 testing:=True;
 lsCmdStr:=' ';
 if not pwdOk then
  begin
   loginForm:=TLoginForm.Create(Self);
   loginForm.NetworkIdEdit.Text:=
        UsernameEdit.Text + '@' +HostEdit.Text;
   if (loginForm.ShowModal = mrOK) then
    begin
     pwdStr:=loginForm.PwdEdit.Text;
     pwdOk:=True;
    end;
   loginForm.Free;
  end;
 if (Length(PortEdit.Text) > 0) then
   lsCmdStr:=' -P ' + PortEdit.Text +' ';
 if not CheckBoxHideConsole.Checked then
   lsCmdStr:=lsCmdStr + '-v ';
 lsCmdStr:=lsCmdStr + '-ls -gui ' +IntToStr(Handle)
               +' -pw ' + pwdStr +' "';
 if (Length(UsernameEdit.Text) > 0) then
   lsCmdStr:=lsCmdStr+ UsernameEdit.Text + '@';
 lsCmdStr:=lsCmdStr+HostEdit.Text;
 if (Pos(':',HostEdit.Text) = 0) then
   lsCmdStr:=lsCmdStr+':';
 lsCmdStr:=lsCmdStr+'"';
 StrPCopy(tempCmd,'pscp');
 StrPCopy(tempParam,lsCmdStr);
 ListViewTest.Items.Clear;
 noResponseCnt:=SpinEditTimeout.Value;
 Screen.Cursor := crHourGlass;
 consoleMode:=SW_NORMAL;
 if CheckBoxHideConsole.Checked then
   consoleMode:=SW_HIDE;
 ShellExecute(0, Nil, tempCmd, tempParam, Nil, consoleMode);
end;

procedure TSetupForm.SpinEditFilePosChange(Sender: TObject);
begin
 ButtonTestClick(Sender);
end;

procedure TSetupForm.NoResponseTimerTimer(Sender: TObject);
begin
 if (noResponseCnt = 1) then
  begin
   Screen.Cursor := crDefault;
   testing:=False;
   noResponseCnt:=0;
   Application.MessageBox('SSH Host Timeout','No response from host',MB_OK);
  end
 else if (noResponseCnt > 1) then
   Dec(noResponseCnt);
end;

procedure TSetupForm.FormCreate(Sender: TObject);
begin
 noResponseCnt := -1;
end;

procedure TSetupForm.FormShow(Sender: TObject);
begin
 SpinEditFilePos.Enabled:=CheckBoxFixPos.Checked;
end;

procedure TSetupForm.CheckBoxFixPosClick(Sender: TObject);
begin
 SpinEditFilePos.Enabled:=CheckBoxFixPos.Checked;
end;

procedure TSetupForm.ButtonExamplesClick(Sender: TObject);
var
 str: String;
begin
str:='Options:' +
#13#10 +
#13#10 +
'- Specify the hostname only:' +
#13#10 +
'   myhost.com' +
#13#10 +
#13#10 +
'- Specify a subdomain and the hostname:' +
#13#10 +
'   subdomain.myhost.com' +
#13#10 +
#13#10 +
'- Specify a subdomain, the hostname and a location on the host:' +
#13#10 +
'   subdomain.myhost.com:myLocation/SampleDir/' +
#13#10 +
#13#10 +
'- Specify the hostname and the root location on the host (instead of home dir):' +
#13#10 +
'   myhost.com:/';
  Application.MessageBox(PChar(str),'Examples of Host Specifications',MB_OK);
end;

end.
