unit UiXplorer;
(*
Secure iXplorer, Copyright  2001 Lars Gunnarsson (lgunnars@i-tree.org).

This software is a windows front end for PSCP.

PSCP is distributed as a part
of PuTTY (http://www.chiark.greenend.org.uk/~sgtatham/putty/)
PuTTY is copyright 1997-1999 Simon Tatham. Portions copyright Gary S. Brown
and Eric Young.

The Drag and Drop Component Suite used within this software is freeware
Copyright  1997-99 by Angus Johnson (ajohnson@rpi.net.au)
and Anders Melander (anders@melander.dk -homesite: http://www.melander.dk).
All rights reserved.

The TdfsExtListView and the SystemControlPack components used within this software are freeware
Copyright  1999-2000, Brad Stowers (bstowers@delphifreestuff.com
-homesite: http://www.delphifreestuff.com). All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Or see: http://www.gnu.org/copyleft/gpl.html

*)

interface

uses
  ExtCtrls, Buttons, ShellAPI, PrevInstance,
  Windows, Messages, SysUtils, Classes, Graphics,
  Controls, Forms, Dialogs, contnrs,
  ToolWin, Psock, NMHttp, StdCtrls, 
  Menus, FileCtrl, ImgList, ComCtrls, DropSource,
  USetup, UHostEntry, SyncObjs, UNewFolder,
  DropTarget, ULsCmdItem, SystemControlPack, SystemListView,
  SystemTreeView, SystemComboBox, 
  ItemProp, Registry, ULoginForm, UPwd,
  Clipbrd, DropPIDLTarget, EnhListView;

const
  regEntryStr = 'iXplorer';
  versionStr = '0.9';
  sectionGeneral = 'General';
  sshPrefix = 'ssh://';
  nonSshFileErrStr = 'One or more files were dragged from a non SSH host';
  errStr2 = 'Secure iXplorere can only copy from SSH hosts';
  noSshSelectionErrStr
   = 'Please select and log in to a SSH location before copying';
  errStr3 = 'No SSH host selected';

const
  WM_STD_OUT_CHAR     = WM_APP + 400;
  WM_STD_ERR_CHAR     =	WM_APP + 401;
  WM_STATS_CHAR	      =	WM_APP + 402;
  WM_STATS_SIZE       =	WM_APP + 403;
  WM_STATS_PERCENT    =	WM_APP + 404;
  WM_STATS_ELAPSED    =	WM_APP + 405;
  WM_RET_ERR_CNT      = WM_APP + 406;
  WM_LS_RET_ERR_CNT   = WM_APP + 407;

type
  TRefMode = (LocNormalChild, LocExtLink, LocShortcut);

  TiXplorerForm = class(TForm)
    ImageListDefault: TImageList;
    Panel1: TPanel;
    SaveDialog1: TSaveDialog;
    PopupMenuMain: TPopupMenu;
    ExitMenu: TMenuItem;
    KeepMenu: TMenuItem;
    AboutMenu: TMenuItem;
    CoolBar1: TCoolBar;
    ToolBar1: TToolBar;
    RemoteToolButton: TToolButton;
    OpenDialog1: TOpenDialog;
    CoolBar2: TCoolBar;
    ToolBar2: TToolBar;
    RmtAddHostToolBtn: TToolButton;
    Panel2: TPanel;
    dfsSystemComboBox1: TdfsSystemComboBox;
    DropFileTarget1: TDropFileTarget;
    DropFileTarget2: TDropFileTarget;
    DropFileSource1: TDropFileSource;
    DropFileSource2: TDropFileSource;
    CopyListView: TListView;
    Panel3: TPanel;
    CopyCmdTimer: TTimer;
    NoResponseTimer: TTimer;
    RemoteMenu: TMenuItem;
    PopupMenuSshTree: TPopupMenu;
    MenuItemTreeViewProperty: TMenuItem;
    RmtDelHostToolBtn: TToolButton;
    ImageListBtn: TImageList;
    ImageListHotBtn: TImageList;
    RmtUpToolBtn: TToolButton;
    RmtCopyToolBtn: TToolButton;
    ToolButton5: TToolButton;
    RmtPasteToolBtn: TToolButton;
    ToolButton7: TToolButton;
    RmtViewToolBtn: TToolButton;
    CoolBar4: TCoolBar;
    ToolBar4: TToolBar;
    LocUpToolBtn: TToolButton;
    LocAddFolderToolBtn: TToolButton;
    ToolButton11: TToolButton;
    LocCopyToolBtn: TToolButton;
    LocPasteToolBtn: TToolButton;
    ToolButton14: TToolButton;
    LocViewToolBtn: TToolButton;
    PopupMenuRmtView: TPopupMenu;
    MenuItemRmtLarge: TMenuItem;
    MenuItemRmtSmall: TMenuItem;
    MenuItemRmtList: TMenuItem;
    MenuItemRmtDetails: TMenuItem;
    MenuItemLocLarge: TMenuItem;
    MenuItemLocSmall: TMenuItem;
    MenuItemLocList: TMenuItem;
    MenuItemLocDetails: TMenuItem;
    MenuItemLocPaste: TMenuItem;
    MenuItemRmtLargeMain: TMenuItem;
    MenuItemRmtSmallMain: TMenuItem;
    MenuItemRmtListMain: TMenuItem;
    MenuItemRmtDetailsMain: TMenuItem;
    MenuItemLocLargeMain: TMenuItem;
    MenuItemLocSmallMain: TMenuItem;
    MenuItemLocListMain: TMenuItem;
    MenuItemLocDetailsMain: TMenuItem;
    PopupMenuLocView: TPopupMenu;
    MenuItemRmtAddHost: TMenuItem;
    MenuItemRmtUp: TMenuItem;
    MenuItemRmtDelHost: TMenuItem;
    MenuItemRmtView: TMenuItem;
    MenuItemRmtCopy: TMenuItem;
    MenuItemRmtPaste: TMenuItem;
    MenuItemLocView: TMenuItem;
    MenuItemOpenNode: TMenuItem;
    HiddenPopupMenu: TPopupMenu;
    MenuItemOpen: TMenuItem;
    ToolButtonHelp: TToolButton;
    MenuItemHelp: TMenuItem;
    StatusPanel: TPanel;
    ProgressBar: TProgressBar;
    ImageListDefaultBig: TImageList;
    CopyQueuePopupMenu: TPopupMenu;
    ClearQueueMenuItem: TMenuItem;
    DownloadToolButton: TToolButton;
    UploadToolButton: TToolButton;
    procedure WndProc(var Msg : TMessage); override;
    procedure MessageHandler(var Msg: TMsg; var Handled: Boolean);
    procedure UpdateIcon(val: Integer; modify: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormExit(Sender: TObject);
    procedure DropFileTarget1Drop(Sender: TObject; ShiftState: TShiftState;
      Point: TPoint; var Effect: Integer);
    procedure ExitMenuClick(Sender: TObject);
    procedure KeepMenuClick(Sender: TObject);
    procedure AboutMenuClick(Sender: TObject);
    procedure DirTreeViewOnMouseDown(Sender: TObject;
                        Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure DirTreeViewChange(Sender: TObject; Node: TTreeNode);
    procedure DirTreeViewChanging(Sender: TObject; Node: TTreeNode;
      var AllowChange: Boolean);
    procedure FormResize(Sender: TObject);
    procedure FileListViewMouseMove(Sender: TObject; Shift: TShiftState; X,Y: Integer);
    procedure SystemListViewMouseMove(Sender: TObject; Shift: TShiftState; X,Y: Integer);
    procedure DropFileTarget2Drop(Sender: TObject; ShiftState: TShiftState;
      Point: TPoint; var Effect: Integer);
    procedure CopyCmdTimerTimer(Sender: TObject);
    procedure FileListViewDblClick(Sender: TObject);
    procedure RmtViewToolBtnClick(Sender: TObject);
    procedure LocViewToolBtnClick(Sender: TObject);
    procedure MenuItemLocLargeClick(Sender: TObject);
    procedure MenuItemLocListClick(Sender: TObject);
    procedure MenuItemLocDetailsClick(Sender: TObject);
    procedure MenuItemLocSmallClick(Sender: TObject);
    procedure MenuItemRmtDetailsClick(Sender: TObject);
    procedure MenuItemRmtLargeClick(Sender: TObject);
    procedure MenuItemRmtListClick(Sender: TObject);
    procedure MenuItemRmtSmallClick(Sender: TObject);
    procedure RmtAddHostToolBtnClick(Sender: TObject);
    procedure MenuItemTreeViewPropertyClick(Sender: TObject);
    procedure MenuItemOpenNodeClick(Sender: TObject);
    procedure NoResponseTimerTimer(Sender: TObject);
    procedure RmtDelHostToolBtnClick(Sender: TObject);
    procedure RmtUpToolBtnClick(Sender: TObject);
    procedure LocUpToolBtnClick(Sender: TObject);
    procedure LocAddFolderToolBtnClick(Sender: TObject);
    procedure MenuItemOpenClick(Sender: TObject);
    procedure HelpMenuClick(Sender: TObject);
    procedure HomePageMenuClick(Sender: TObject);
    procedure RmtCopyToolBtnClick(Sender: TObject);
    procedure LocCopyToolBtnClick(Sender: TObject);
    procedure RmtPasteToolBtnClick(Sender: TObject);
    procedure LocPasteToolBtnClick(Sender: TObject);
    procedure ClearQueueMenuItemClick(Sender: TObject);
    procedure DownloadToolButtonClick(Sender: TObject);
    procedure UploadToolButtonClick(Sender: TObject);
  private
    procedure SetLocViewBtn(inx: Integer);
    procedure SetRmtViewBtn(inx: Integer);
    function PasswordFound(const id: String; var retPwd: TiXplorerPwd): Boolean;
    function PasswordStringFound(const id: String; var retStr: String): Boolean;
    function GetPassword(const id: String; var ok, usedDialog: Boolean): String;
    procedure WriteRegistry();
    procedure HostWriteRegistry(const idStr: String;
                                const itemRec: TiXplorerHostEntry);
    procedure ReadRegistry();
    procedure GetHostEntry(const id: String; var ret: TiXplorerHostEntry);
    procedure OpenObject(sObjectPath : PChar);
    function GetTimeStringFromSeconds(secs: Integer): String;
    procedure SelectDirTreeNode(Node: TTreeNode; canBeSame: Boolean);
    function ReplaceSpecialChar(const inStr: String) : String;
    function DoubleUpTrailingBackslash(const inStr: String) : String;
    procedure FileViewKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure SystemViewKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
  private
    IconData : TNotifyIconData;
    prevIconVal: Integer;
    statStr,copyingNowStr,lineStr: String;
    selDirStr,selHostStr: String;
    selNodeStr,curTargetStr: String;
    selNode,parentNode,mouseCurNode: TTreeNode;
    bLoading,bChildrenExists: Boolean;
    bCopyingInProgress: Boolean;
    DirTreeView: TTreeView;
    FileListView: TdfsEnhListView;
    Split1,Split2: TSplitter;
    systemTreeView: TdfsSystemTreeView;
    systemListView: TdfsSystemListView;
    lockListAccess: TCriticalSection;
    pwdList: TObjectList;
    fIniFile: TRegIniFile;
    hostList: TStringList;
    noResponseCnt: Integer;
    copyTimeoutCnt: Integer;
    cancelCopy, pwdEntered: Boolean;
    pwdItem: TiXplorerPwd;
    curHostEntry,curCopyHostEntry: TiXplorerHostEntry;
    curCopySize,elapsedTime: Integer;
  protected
    procedure WMStdOutCharReceived(var Message: TMessage);     message WM_STD_OUT_CHAR;
    procedure WMStdErrCharReceived(var Message: TMessage);     message WM_STD_ERR_CHAR;
    procedure WMErrCntReceived(var Message: TMessage);         message WM_RET_ERR_CNT;
    procedure WMLsCmdErrCntReceived(var Message: TMessage);    message WM_LS_RET_ERR_CNT;
    procedure WMStatsCharReceived(var Message: TMessage);      message WM_STATS_CHAR;
    procedure WMStatsSizeReceived(var Message: TMessage);      message WM_STATS_SIZE;
    procedure WMStatsPercentReceived(var Message: TMessage);   message WM_STATS_PERCENT;
    procedure WMStatsElapsedReceived(var Message: TMessage);   message WM_STATS_ELAPSED;
  private
    procedure Load(orgNode: TTreeNode);
    procedure CancelCopying;
  end;

var
  AlreadyDragging: boolean;
  DragPoint: TPoint;
  iXplorerForm: TiXplorerForm;

implementation

{$R *.DFM}

procedure TiXplorerForm.WMErrCntReceived(var Message: TMessage);
var
 cnt: Integer;
begin
 lockListAccess.Acquire; { lock out other threads }
 try
   cnt:=CopyListView.Items.Count;
 finally
   lockListAccess.Release;
 end;
 if (cnt > 0) and not cancelCopy then
  begin
   lockListAccess.Acquire; { lock out other threads }
   try
     CopyListView.Items.Delete(0);
   finally
     lockListAccess.Release;
   end;
  end;
 // Finished copying
 if (curTargetStr = (selHostStr+selDirStr)) then
   SelectDirTreeNode(selNode,True);
// New in Version 0.8
 if (ExcludeTrailingBackslash(curTargetStr)
     = ExcludeTrailingBackslash(systemTreeView.Directory)) then
   systemTreeView.Reset;
 ProgressBar.Position:=0;
 StatusPanel.Caption:= 'Copied ' +copyingNowStr +': '
               +IntToStr(curCopySize) +' bytes in '
               +IntToStr(elapsedTime) +' second';
 if (elapsedTime <> 1) then
   StatusPanel.Caption:=StatusPanel.Caption+'s';
 copyTimeoutCnt:=-1;
 bCopyingInProgress:=False;
end;

procedure TiXplorerForm.WMLsCmdErrCntReceived(var Message: TMessage);
begin
 Screen.Cursor := crDefault;
 bLoading:=False;
 pwdItem.VerifyPasswordIsOK;
 pwdEntered:=False;
 noResponseCnt:=-1;
end;

procedure TiXplorerForm.WMStdOutCharReceived(var Message: TMessage);
begin
 WMStdErrCharReceived(Message);
end;

procedure TiXplorerForm.WMStdErrCharReceived(var Message: TMessage);
var
 ch: char;
 ListItem: TListItem;
 lsItem: TLsCmdItem;
 curNode: TTreeNode;
 size: Integer;
 nameStr: String;
begin
 ch := Chr(Message.WParam);
 if (ch <> #10) then
   lineStr:=lineStr+ch
 else
  begin
   if curHostEntry.FixPositionIsSet then
     lsItem:=TLsCmdItem.Create(lineStr,curHostEntry.GetFilepos)
   else
     lsItem:=TLsCmdItem.Create(lineStr,-1);
   with lsItem do
    if IsValidLine() then
     begin
      nameStr:=GetName;
      if (Length(nameStr) > 0) and (nameStr <> '.') and (nameStr <> '..') then
       begin
        ListItem := FileListView.Items.Add;
        ListItem.Caption := GetLsStr;
        size:=GetFileSize;
        if (size >= 0) then
          ListItem.SubItems.Add(IntToStr(size))
        else
          ListItem.SubItems.Add('-');
        ListItem.SubItems.Add(GetDateStr);
        ListItem.SubItems.Add(GetPermissionsStr);
        ListItem.SubItems.Add(GetOwnerStr);
        ListItem.SubItems.Add(GetGroupStr);
        size:=GetINodeBlocks;
        if (size >= 0) then
          ListItem.SubItems.Add(IntToStr(size))
        else
          ListItem.SubItems.Add('-');
        if IsDirectory() then
         begin
          if not bChildrenExists then
           begin
            curNode:=DirTreeView.Items.AddChild(selNode, nameStr);
            curNode.ImageIndex:=1;
            curNode.SelectedIndex:=2;
           end;
          ListItem.ImageIndex:=1;
         end
        else if lsItem.IsLink() then
          ListItem.ImageIndex:=4
        else
          ListItem.ImageIndex:=3;
       end;
     end;
   lsItem.Free;
   lineStr:='';
  end;
end;

procedure TiXplorerForm.WMStatsCharReceived(var Message: TMessage);
var
 ch: char;
begin
 ch := Chr(Message.WParam);
 if (ch <> #10) then
   statStr:=statStr+ch
 else
  begin
   copyingNowStr:=statStr;
   statStr:='';
  end;
end;

procedure TiXplorerForm.WMStatsSizeReceived(var Message: TMessage);
begin
 curCopySize:=Integer(Message.WParam);
end;

procedure TiXplorerForm.WMStatsPercentReceived(var Message: TMessage);
begin
 ProgressBar.Position:=Integer(Message.WParam);
end;

function TiXplorerForm.GetTimeStringFromSeconds(secs: Integer): String;
var
 tmpHour,tmpMin,tmpSec: Integer;
 retStr: String;
begin
 retStr:='';
 if (secs < 60) then
  begin
   retStr:=IntToStr(secs) +' second';
   if (secs <> 1) then
     retStr:=retStr+'s';
  end
 else
  begin
   tmpSec:=secs mod 60;
   tmpMin:=secs div 60;
   tmpHour:=tmpMin div 60;
   Dec(tmpMin,(tmpHour*60));
   retStr:=Format('%d:%.2d:%.2d', [tmpHour,tmpMin,tmpSec]);
  end;
 GetTimeStringFromSeconds:=retStr;
end;

procedure TiXplorerForm.WMStatsElapsedReceived(var Message: TMessage);
var
 percent,remain: Integer;
begin
 copyTimeoutCnt:=curCopyHostEntry.GetTimeout; // Resetting the timeout
 elapsedTime:=Integer(Message.WParam);
 StatusPanel.Caption:=copyingNowStr +': '
               +IntToStr(ProgressBar.Position) + ' % of '
               +IntToStr(curCopySize) +' bytes in '
               +GetTimeStringFromSeconds(elapsedTime);
 percent:=ProgressBar.Position;
 if (percent > 0) and (elapsedTime > 0) then
  begin
   remain:=((100-percent) * elapsedTime) div percent;
   StatusPanel.Caption:=StatusPanel.Caption
                +', Approx. ' +GetTimeStringFromSeconds(remain) +' remaining';
  end;
end;

procedure TiXplorerForm.Load(orgNode: TTreeNode);
var
 i: Integer;
 tNode: TTreeNode;
begin
 if (orgNode = nil) then
  begin
   DirTreeView.Items.Clear;
   selNode:=nil;
   selDirStr:='';
   selHostStr:='';
   selNodeStr:='';
   parentNode:=nil;
   tNode := DirTreeView.Items.Add(orgNode, 'SSH Hosts'); {Add a root node }
   for i:=0 to hostList.Count-1 do
    begin
     DirTreeView.Items.AddChild(tNode, hostList[i]);
    end;
  end;
end;

procedure TiXplorerForm.FormCreate(Sender: TObject);
var
  NewColumn: TListColumn;
begin
 pwdList:=TObjectList.Create;
 hostList:=TStringList.Create;
 fIniFile := TRegIniFile.Create(regEntryStr);
 Application.OnMessage := MessageHandler;
 prevIconVal := -1;
 noResponseCnt := -1;
 IconData.cbSize := sizeof(IconData);
 IconData.Wnd := Handle;
 IconData.uID := 100;
 IconData.uFlags := NIF_MESSAGE + NIF_ICON + NIF_TIP;
 IconData.uCallbackMessage := WM_USER + 1;
 StrPCopy(IconData.szTip, Application.Title);
 UpdateIcon(0,False);
 curHostEntry:=TiXplorerHostEntry.Create;
 curCopyHostEntry:=TiXplorerHostEntry.Create;
 lockListAccess:=TCriticalSection.Create;
 Panel1.Width:=iXplorerForm.ClientWidth div 2;
 DirTreeView := TTreeView.Create(iXplorerForm);
 { line it up on the left of the form }
 with DirTreeView do
  begin
   Parent := Panel1;
   Align := alLeft;
   Width := Panel1.Width div 3;
   OnMouseDown:=DirTreeViewOnMouseDown;
   OnChange:=DirTreeViewChange;
   OnChanging:=DirTreeViewChanging;
   OnKeyDown:=FileViewKeyDown;
   PopUpMenu:=PopupMenuSshTree;
   Images:=ImageListDefault;
  end;
{ now add the splitter to divide the directory pane from the file pane }
 Split1 := TSplitter.Create(iXplorerForm);
 Split1.Parent := Panel1;
 Split1.Left := DirTreeView.Left + DirTreeView.Width + 1;
 Split1.Align := DirTreeView.Align;
 Split1.MinSize := Panel1.ClientWidth div 4;
 Split1.AutoSnap:=False;
 RmtViewToolBtn.ImageIndex:=8;
 FileListView := TdfsEnhListView.Create(iXplorerForm);
 with FileListView do
  begin
   Parent := Panel1;
   Align := alClient;
   SmallImages:=ImageListDefault;
   LargeImages:=ImageListDefaultBig;
   ViewStyle:=vsList;
   NewColumn := Columns.Add;
   NewColumn.Caption := 'Name';
   NewColumn.Width := 140;
   NewColumn := Columns.Add;
   NewColumn.Caption := 'Size';
   NewColumn := Columns.Add;
   NewColumn.Caption := 'Date';
   NewColumn.Width := 100;
   NewColumn := Columns.Add;
   NewColumn.Caption := 'Permissions';
   NewColumn.Width := 90;
   NewColumn := Columns.Add;
   NewColumn.Caption := 'Owner';
   NewColumn.Width := 70;
   NewColumn := Columns.Add;
   NewColumn.Caption := 'Group';
   NewColumn.Width := 70;
   NewColumn := Columns.Add;
   NewColumn.Caption := 'inode';
   MultiSelect:=True;
   OnMouseMove:=FileListViewMouseMove;
   OnDblClick:=FileListViewDblClick;
   OnKeyDown:=FileViewKeyDown;
   AutoColumnSort:=acsSortToggle;
   AutoSortStyle:=assDefault;
   NoColumnResize:=False;
   ShowSortArrows:=True;
   SaveSettings.AutoSave:=True;
   SaveSettings.RegistryKey:='\Software\i-tree.org\iXplorer\EnhListView';
  end;

 systemTreeView := TdfsSystemTreeView.Create(iXplorerForm);
 with systemTreeView do
  begin
   Parent := Panel2;
   Align := alLeft;
   Width := Panel2.Width div 2;
   ListView := systemListView;
   OnKeyDown:=SystemViewKeyDown;
  end;
{ now add the splitter to divide the directory pane from the file pane }
 Split2 := TSplitter.Create(iXplorerForm);
 Split2.Parent := Panel2;
 Split2.Left := systemTreeView.Left + systemTreeView.Width + 1;
 Split2.Align := systemTreeView.Align;
 Split2.MinSize := Panel2.ClientWidth div 4;
 Split2.AutoSnap:=False;
 LocViewToolBtn.ImageIndex:=8;
 systemListView := TdfsSystemListView.Create(iXplorerForm);
 with systemListView do
  begin
   Parent := Panel2;
   Align := alClient;
   ViewStyle:=vsList;
   TreeView := systemTreeView;
   MultiSelect:=True;
   OnMouseMove:=SystemListViewMouseMove;
   OnKeyDown:=SystemViewKeyDown;
//   OnDeletion:=SystemListViewDeletion;
  end;

 ReadRegistry; 
 Load(nil);
 DropFileTarget1.register(FileListView);
 DropFileTarget2.register(systemListView);
end;

procedure TiXplorerForm.FormDestroy(Sender: TObject);
begin
 pwdList.Free;
 hostList.Free;
 fIniFile.Free;
 lockListAccess.Free;
 curHostEntry.Free;
 curCopyHostEntry.Free;
 DropFileTarget1.unregister;
 DropFileTarget2.unregister;
 DirTreeView.Free;
 FileListView.Free;
 Split1.Free;
 Split2.Free;
 systemTreeView.Free;
 systemListView.Free;
end;

procedure TiXplorerForm.UpdateIcon(val: Integer; modify: Boolean);
var
  tempIcon: TIcon;
begin
  tempIcon := TIcon.Create;
  ImageListDefault.GetIcon(val,tempIcon);
  IconData.hIcon := tempIcon.Handle;
  if (modify) then
    Shell_NotifyIcon(NIM_MODIFY, @IconData)
  else
    Shell_NotifyIcon(NIM_ADD, @IconData);
  tempIcon.Free;
end;

procedure TiXplorerForm.MessageHandler(var Msg: TMsg; var Handled: Boolean);
var
  s : string;
begin
  if Msg.message = ThisInstanceMessageId then
   begin
    Handled := True;
    s := StrPas(PChar(Msg.LParam));
    // We do not actually deal with the command line parameter here
    // Might be useful in the future, though...
    SetForegroundWindow(Handle);
    Show;
    ShowWindow(Application.Handle, SW_HIDE);
    PostMessage(Handle, WM_NULL, 0, 0);
   end;
  { for all other messages, Handled remains False }
  { so that other message handlers can respond }
end;

procedure TiXplorerForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
 Action := caNone;
 Hide;
 if not KeepMenu.Checked then
   FormExit(Self);
end;

procedure TiXplorerForm.FormExit(Sender: TObject);
begin
 Shell_NotifyIcon(NIM_DELETE, @IconData);
 Application.ProcessMessages;
 Application.Terminate;
end;

procedure TiXplorerForm.WndProc(var Msg : TMessage);
var
  p : TPoint;
begin
  case Msg.Msg of
    WM_USER + 1:
    case Msg.lParam of
      WM_RBUTTONDOWN: begin
         SetForegroundWindow(Handle);
         GetCursorPos(p);
         PopupMenuMain.Popup(p.x, p.y);
         PostMessage(Handle, WM_NULL, 0, 0);
      end;
      WM_LBUTTONDOWN: begin
         SetForegroundWindow(Handle);
         Show;
         ShowWindow(Application.Handle, SW_HIDE);
         PostMessage(Handle, WM_NULL, 0, 0);
      end
    end;
  end;
  inherited;
end;

procedure TiXplorerForm.ExitMenuClick(Sender: TObject);
begin
 FormExit(Self);
end;

procedure TiXplorerForm.KeepMenuClick(Sender: TObject);
begin
 KeepMenu.Checked:= not KeepMenu.Checked;
 WriteRegistry();
end;

procedure TiXplorerForm.AboutMenuClick(Sender: TObject);
var
 str: String;
begin
str:='Secure iXplorer, version '+versionStr+', Copyright  2001, ' +
' Lars Gunnarsson (lgunnars@i-tree.org),' +
#13#10 +
'This software is a windows front end for PSCP.' +
#13#10 +
#13#10 +
'PSCP is distributed as a part ' +
'of PuTTY (http://www.chiark.greenend.org.uk/~sgtatham/putty/)' +
#13#10 +
'PuTTY is copyright 1997-1999 Simon Tatham. Portions copyright Gary S. Brown and Eric Young.' +
#13#10 +
#13#10 +
'The Drag and Drop Component Suite used within this software is freeware,' +
#13#10 +
'Copyright  1997-99 by Angus Johnson (ajohnson@rpi.net.au)'  +
#13#10 +
'and Anders Melander (anders@melander.dk -homesite: http://www.melander.dk). All rights reserved.' +
#13#10 +
#13#10 +
'The TdfsExtListView and the SystemControlPack components used within this software are freeware,' +
#13#10 +
'Copyright  1999-2000, Brad Stowers. (bstowers@delphifreestuff.com -homesite: http://www.delphifreestuff.com).' +
#13#10 +
' All rights reserved.' +
#13#10 +
#13#10 +
'This program comes with ABSOLUTELY NO WARRANTY. '
+ #13#10 +
'It is free software, and you are welcome to redistribute it under ' +
'certain conditions;' +
#13#10 +
'See the GNU General Public License at ' +
'http://www.gnu.org/copyleft/gpl.html' +
' for more details.';
  Application.MessageBox(PChar(str),'About i-tree.org Secure iXplorer Version '+versionStr,MB_OK);
end;

procedure TiXplorerForm.DirTreeViewOnMouseDown(Sender: TObject;
                        Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 if (Button = mbRight) then
   mouseCurNode:=DirTreeView.GetNodeAt(X, Y);
end;

procedure TiXplorerForm.SelectDirTreeNode(Node: TTreeNode; canBeSame: Boolean);
var
  found,updateLocalDir: Boolean;
  lsCmdStr,pwStr: String;
  colonPos,tmpLen,consoleMode: Integer;
  tempCmd,tempParam: array[0..10000] of char;
begin
  if bLoading or ((DirTreeView.Selected = selNode) and not canBeSame) then Exit;
  selNode:=DirTreeView.Selected;
  selDirStr:='';
  selHostStr:='';
  selNodeStr:='';
  if (selNode = nil) then Exit;
  if (selNode.Level >= 1) then
   begin
    Screen.Cursor := crHourGlass;
    bLoading:=True;

// New in Version 0.9 - Refresh the whole subtree
    if (DirTreeView.Selected = selNode) then
      selNode.DeleteChildren;

    bChildrenExists:=(selNode.Count>0);
    parentNode:=selNode;
    while (parentNode.Level > 1) do
     begin
      if (Length(selDirStr) > 0) then
        selDirStr:='/'+selDirStr;
      selDirStr:=parentNode.Text+selDirStr;
      parentNode:=parentNode.Parent;
     end;
    if (Length(selDirStr) > 0) then
      selDirStr:=selDirStr+'/';
    selNodeStr:=parentNode.Text;
    pwStr:=GetPassword(parentNode.Text, found, updateLocalDir);
    if not found then
     begin
      bLoading:=False;
      Screen.Cursor := crDefault;
     end
    else
     begin
      GetHostEntry(parentNode.Text,curHostEntry);
// This is not checked at the moment -might be an option later
//      if updateLocalDir and
      if (selNode.Level = 1) and (Length(curHostEntry.GetLocalDirPath) > 0) then
       begin
        if DirectoryExists(curHostEntry.GetLocalDirPath) then
          systemTreeView.Directory:=curHostEntry.GetLocalDirPath
        else
          Application.MessageBox(PChar(curHostEntry.GetLocalDirPath +' does not exist!'),
            'Error: Setting Local Directory Path',MB_OK);
       end;
      selHostStr:= curHostEntry.GetUserName +'@' + curHostEntry.GetHostPath;
      colonPos:=Pos(':',selHostStr);
      tmpLen:=Length(selHostStr);
      if (colonPos = 0) then
        selHostStr:=selHostStr+':'
      else if (colonPos < tmpLen) and (tmpLen > 0) and (selHostStr[tmpLen] <> '/')then
        selHostStr:=selHostStr+'/';
      lsCmdStr:=' ';
      if (Length(curHostEntry.GetPortNumStr) > 0) then
        lsCmdStr:=' -P ' + curHostEntry.GetPortNumStr +' ';
      if not curHostEntry.HideConsoleIsSet then
        lsCmdStr:=lsCmdStr + '-v ';
      lsCmdStr:=lsCmdStr + '-ls -gui ' +IntToStr(Handle)
               +' -pw ' + pwStr
               +' "' + selHostStr + selDirStr +'"';
      StrPCopy(tempCmd,'pscp');
      StrPCopy(tempParam,lsCmdStr);
      FileListView.Items.Clear;
      noResponseCnt:=curHostEntry.GetTimeout;
      consoleMode:=SW_NORMAL;
      if curHostEntry.HideConsoleIsSet then
        consoleMode:=SW_HIDE;
      ShellExecute(0, Nil, tempCmd, tempParam, Nil, consoleMode);
     end;
   end;
end;

procedure TiXplorerForm.DirTreeViewChange(Sender: TObject; Node: TTreeNode);
begin
  SelectDirTreeNode(Node,False);
end;

procedure TiXplorerForm.DirTreeViewChanging(Sender: TObject; Node: TTreeNode;
  var AllowChange: Boolean);
begin
  AllowChange:=not bLoading;
end;

procedure TiXplorerForm.FormResize(Sender: TObject);
begin
 if Panel1 <> nil then
   Panel1.Width := Self.ClientWidth div 2;
 if Panel2 <> nil then
   Panel2.Width := Self.ClientWidth div 2;
 if FileListView <> nil then
   FileListView.Width := Panel1.Width;
 if systemListView <> nil then
   systemListView.Width := Panel2.Width;
end;

procedure TiXplorerForm.FileListViewMouseMove(Sender: TObject;
    Shift: TShiftState; X,Y: Integer);
var
  i: integer;
  prefixStr: String;
begin
  if AlreadyDragging then exit;
  //Filter out all but the mouse buttons from 'Shift'...
  Shift := (Shift * [ssLeft,ssRight]);
  //Make sure a mouse button is pressed before starting the drag ...
  //and the mouse has moved at least 10 pixels
  //(DragPoint is set in the ListView1MouseDown event)
  if  (Shift = []) or
      ((abs(DragPoint.X - X) <10) and (abs(DragPoint.Y - Y) <10)) then
    exit;
  //If no files selected then exit...
  if FileListView.SelCount = 0 then exit;
  //Delete any filenames left from a previous dragdrop...
  DropFileSource1.Files.clear;
  //Fill DropSource1.Files with selected filenames in ListView1...
  if Length(selDirStr) > 0 then
    prefixStr:=sshPrefix+selHostStr+selDirStr
  else
    prefixStr:=sshPrefix+selHostStr;
  for i := 0 to FileListView.items.Count-1 do
    if (FileListView.items[i].Selected)
    // To Do!!! Implement handling links -> presently they are disabled here
    and (FileListView.items[i].ImageIndex <> 4) then
      DropFileSource1.Files.Add(prefixStr+FileListView.items.item[i].caption);
  //Stop this event being re-executed once a drag has been initiated...
  AlreadyDragging := true;
  //Start the dragdrop...
  //Note: DropFileSource1.DragTypes = [dtCopy] set at design time.
  DropFileSource1.execute;
  //Note: Execute does not return until the drop has finished -
  //either sucessfully with a copy,move of link operation or cancelled.
  AlreadyDragging := false;
end;

procedure TiXplorerForm.SystemListViewMouseMove(Sender: TObject;
    Shift: TShiftState; X,Y: Integer);
var
  i: integer;
begin
  if AlreadyDragging then exit;
  //Filter out all but the mouse buttons from 'Shift'...
  Shift := (Shift * [ssLeft,ssRight]);
  //Make sure a mouse button is pressed before starting the drag ...
  //and the mouse has moved at least 10 pixels
  //(DragPoint is set in the ListView1MouseDown event)
  if (Shift = []) or
      ((abs(DragPoint.X - X) <10) and (abs(DragPoint.Y - Y) <10)) then
    exit;
  //If no files selected then exit...
  if SystemListView.SelCount = 0 then exit;
  //Delete any filenames left from a previous dragdrop...
  DropFileSource2.Files.clear;
  //Fill DropSource1.Files with selected filenames in ListView1...
  for i := 0 to SystemListView.items.Count-1 do
    if (SystemListView.items.item[i].Selected) then
      DropFileSource2.Files.Add(
         SystemListView.FullFilename[SystemListView.items.item[i]]);
  //Stop this event being re-executed once a drag has been initiated...
  AlreadyDragging := true;
  //Start the dragdrop...
  //Note: DropFileSource2.DragTypes = [dtCopy] set at design time.
  DropFileSource2.execute;
  //Note: Execute does not return until the drop has finished -
  //either sucessfully with a copy,move of link operation or cancelled.
  AlreadyDragging := false;
end;

procedure TiXplorerForm.DropFileTarget1Drop(Sender: TObject;
  ShiftState: TShiftState; Point: TPoint; var Effect: Integer);
var
 i: integer;
 ListItem: TListItem;
 fNameStr: String;
begin
 if (Length(selNodeStr) > 0) then
  with DropFileTarget1 do
   for i := 0 to files.count-1 do
    begin
     fNameStr:=files[i];
     if (Pos(sshPrefix,fNameStr) <> 1) then
      begin
       lockListAccess.Acquire; // Lock out other threads
       ListItem := CopyListView.Items.Add;
       ListItem.Caption := fNameStr;
       ListItem.SubItems.Add(sshPrefix+selHostStr+selDirStr);
       ListItem.SubItems.Add(selNodeStr);
       lockListAccess.Release;
      end;
    end;
end;

procedure TiXplorerForm.DropFileTarget2Drop(Sender: TObject;
  ShiftState: TShiftState; Point: TPoint; var Effect: Integer);
var
 i: integer;
 ListItem: TListItem;
 fNameStr,targetStr: String;
 nonSshFileFound: Boolean;
begin
 nonSshFileFound:=False;
 if (Length(selNodeStr) > 0) then
  with DropFileTarget2 do
   for i := 0 to files.count-1 do
    begin
     fNameStr:=files[i];
     if (Pos(sshPrefix,fNameStr) <> 1) then
       nonSshFileFound:=True
     else
      begin
       lockListAccess.Acquire; // Lock out other threads
       ListItem := CopyListView.Items.Add;
       ListItem.Caption := fNameStr;
       targetStr:=IncludeTrailingBackslash(SystemTreeView.Directory);
// New in Version 0.8
       if (Length(targetStr)=3) and (targetStr[2] = ':') then
         //This is a disk letter spec and must be without any '\' character
         targetStr:=ExcludeTrailingBackslash(SystemTreeView.Directory);
       ListItem.SubItems.Add(targetStr);
       ListItem.SubItems.Add(selNodeStr);
       lockListAccess.Release;
      end;
    end
 else
   Application.MessageBox(noSshSelectionErrStr,errStr3,MB_OK);
 if nonSshFileFound then
   Application.MessageBox(nonSshFileErrStr,errStr2,MB_OK);
end;

procedure TiXplorerForm.CancelCopying;
begin
 cancelCopy:=True;
 CopyListView.Items.Clear;
 bCopyingInProgress:=False;
 copyTimeoutCnt:=-1;
 cancelCopy:=False;
end;

function TiXplorerForm.ReplaceSpecialChar(const inStr: String) : String;
var
 i: Integer;
 retStr: String;
begin
 for i := 1 to Length(inStr) do
   if inStr[i] = ' ' then
     retStr := retStr+'\ '
// New in Version 0.9
   else if inStr[i] = '''' then
     retStr := retStr+'\'''
   else
     retStr := retStr+inStr[i];
 ReplaceSpecialChar:=retStr;
end;

// New in Version 0.9
function TiXplorerForm.DoubleUpTrailingBackslash(const inStr: String) : String;
var
 strLen: Integer;
 retStr: String;
begin
 strLen:=Length(inStr);
 if (strLen>1) and (inStr[strLen]='\') and (inStr[strLen-1]<>'\') then
   retStr := inStr+'\'
 else
   retStr:=inStr;
 DoubleUpTrailingBackslash:=retStr;
end;

procedure TiXplorerForm.CopyCmdTimerTimer(Sender: TObject);
var
 cnt,consoleMode: Integer;
 sourceStr,
 copyIDStr, lsCmdStr: String;
 tempCmd,tempParam: array[0..10000] of char;
 download, dummy, dummy2: Boolean;
begin
 cnt:=0;
 if bCopyingInProgress and not cancelCopy then
  begin
   if (copyTimeoutCnt > 1) then
     Dec(copyTimeoutCnt)
   else
    begin
     if (copyTimeoutCnt = 1) then
      begin
       copyTimeoutCnt:=-1;
       if (Application.MessageBox('SSH Host Timeout'
             +#13#10
             +'Clear the files in the queue? ',
             'No response from host while copying!',MB_YESNO) = ID_YES) then
         CancelCopying
       else
         copyTimeoutCnt:=curCopyHostEntry.GetTimeout;
      end;
    end;
  end;
 // Check if there are files to copy
 if not bCopyingInProgress and not cancelCopy then
  begin
   lockListAccess.Acquire; { lock out other threads }
   try
     cnt:=CopyListView.Items.Count;
   finally
     lockListAccess.Release;
   end;
  end;
 if (cnt > 0) then
  begin
   bCopyingInProgress :=True;
   lockListAccess.Acquire; { lock out other threads }
   try
     sourceStr:=CopyListView.Items[0].Caption;
     download:=True;
     if (Copy(sourceStr,0,6) = sshPrefix) then
       sourceStr:=Copy(sourceStr,7,Length(sourceStr));
     curTargetStr:=CopyListView.Items[0].SubItems[0];
     if (Copy(curTargetStr,0,6) = sshPrefix) then
      begin
       curTargetStr:=Copy(curTargetStr,7,Length(curTargetStr));
       download:=False; // We are uploading from local to remote
      end;
     copyIDStr:=CopyListView.Items[0].SubItems[1];
   finally
     lockListAccess.Release;
   end;
   lsCmdStr:=' ';
   GetHostEntry(copyIDStr,curCopyHostEntry);
   if curCopyHostEntry.PreserveAttribIsSet then
     lsCmdStr:='-p ';
   if (Length(curCopyHostEntry.GetPortNumStr) > 0) then
     lsCmdStr:=lsCmdStr + '-P ' + curCopyHostEntry.GetPortNumStr +' ';
   if not curCopyHostEntry.HideConsoleIsSet then
     lsCmdStr:=lsCmdStr + '-v ';
   if download then
    begin
// There is a BUG (or feature) in pscp -which makes it necessary
// to add a second trailing backslash to any already present trailing backslash.
// Otherwise the backslash is seen as the opening code for a special
// character, I guess...
// This is only necessary if we enclose the target string with qoutes, which
// we have to do in order to allow spaces in filenames.
//
// We also have to replace any spaces ' ' with '\ ' for the source string
     lsCmdStr:=lsCmdStr + '-gui ' +IntToStr(Handle)
            +' -r -pw ' + GetPassword(copyIDStr, dummy, dummy2)
            +' "' + ReplaceSpecialChar(sourceStr)
            +'" "' + DoubleUpTrailingBackslash(curTargetStr) +'"';
    end
   else
    begin
     if not FileExists(sourceStr) and DirectoryExists(sourceStr) then
       sourceStr:=ExcludeTrailingBackslash(sourceStr);
     lsCmdStr:=lsCmdStr + '-gui ' +IntToStr(Handle)
            +' -r -pw ' + GetPassword(copyIDStr, dummy, dummy2)
            +' "' + sourceStr
            +'" "' + ReplaceSpecialChar(curTargetStr) +'"';
    end;
   StrPCopy(tempCmd,'pscp');
   StrPCopy(tempParam,lsCmdStr);
   consoleMode:=SW_NORMAL;
   if curCopyHostEntry.HideConsoleIsSet then
     consoleMode:=SW_HIDE;
   copyTimeoutCnt:=curCopyHostEntry.GetTimeout;
   ShellExecute(0, Nil, tempCmd, tempParam, Nil, consoleMode);
  end;
end;

procedure TiXplorerForm.FileListViewDblClick(Sender: TObject);
var
 i: Integer;
 checkStr: String;
begin
 if (selNode = nil) or (FileListView.Selected = nil) then Exit;
 checkStr:=FileListView.Selected.Caption;
 if (FileListView.Selected.ImageIndex = 1) then
  begin
   for i:=0 to DirTreeView.Selected.Count-1 do
    if (DirTreeView.Selected.Item[i].Text=checkStr) then
     begin
      DirTreeView.Selected:=selNode.Item[i];
      DirTreeViewChange(Self,DirTreeView.Selected);
      Exit;
     end;
  end
end;

procedure TiXplorerForm.RmtViewToolBtnClick(Sender: TObject);
begin
 if (RmtViewToolBtn.ImageIndex = 9) then
   SetRmtViewBtn(6)
 else
   SetRmtViewBtn(RmtViewToolBtn.ImageIndex+1);
end;

procedure TiXplorerForm.SetRmtViewBtn(inx: Integer);
begin
 RmtViewToolBtn.ImageIndex:=inx;
 MenuItemRmtView.ImageIndex:=inx;
 if (inx = 6) then
  begin
   FileListView.ViewStyle:=vsIcon;
   MenuItemRmtLarge.Checked:=True;
   MenuItemRmtLargeMain.Checked:=True;
  end
 else if (inx = 7) then
  begin
   FileListView.ViewStyle:=vsSmallIcon;
   MenuItemRmtSmall.Checked:=True;
   MenuItemRmtSmallMain.Checked:=True;
  end
 else if (inx = 8) then
  begin
   FileListView.ViewStyle:=vsList;
   MenuItemRmtList.Checked:=True;
   MenuItemRmtListMain.Checked:=True;
  end
 else
  begin
   FileListView.ViewStyle:=vsReport;
   MenuItemRmtDetails.Checked:=True;
   MenuItemRmtDetailsMain.Checked:=True;
  end;
end;

procedure TiXplorerForm.LocViewToolBtnClick(Sender: TObject);
begin
 if (LocViewToolBtn.ImageIndex = 9) then
   SetLocViewBtn(6)
 else
   SetLocViewBtn(LocViewToolBtn.ImageIndex+1);
end;

procedure TiXplorerForm.SetLocViewBtn;
begin
 LocViewToolBtn.ImageIndex:=inx;
 MenuItemLocView.ImageIndex:=inx;
 if (inx = 6) then
  begin
   SystemListView.ViewStyle:=vsIcon;
   MenuItemLocLarge.Checked:=True;
   MenuItemLocLargeMain.Checked:=True;
  end
 else if (inx = 7) then
  begin
   SystemListView.ViewStyle:=vsSmallIcon;
   MenuItemLocSmall.Checked:=True;
   MenuItemLocSmallMain.Checked:=True;
  end
 else if (inx = 8) then
  begin
   SystemListView.ViewStyle:=vsList;
   MenuItemLocList.Checked:=True;
   MenuItemLocListMain.Checked:=True;
  end
 else
  begin
   SystemListView.ViewStyle:=vsReport;
   MenuItemLocDetails.Checked:=True;
   MenuItemLocDetailsMain.Checked:=True;
  end;
end;

procedure TiXplorerForm.MenuItemLocLargeClick(Sender: TObject);
begin
 SetLocViewBtn(6);
end;

procedure TiXplorerForm.MenuItemLocSmallClick(Sender: TObject);
begin
 SetLocViewBtn(7);
end;

procedure TiXplorerForm.MenuItemLocListClick(Sender: TObject);
begin
 SetLocViewBtn(8);
end;

procedure TiXplorerForm.MenuItemLocDetailsClick(Sender: TObject);
begin
 SetLocViewBtn(9);
end;

procedure TiXplorerForm.MenuItemRmtLargeClick(Sender: TObject);
begin
 SetRmtViewBtn(6);
end;

procedure TiXplorerForm.MenuItemRmtSmallClick(Sender: TObject);
begin
 SetRmtViewBtn(7);
end;

procedure TiXplorerForm.MenuItemRmtListClick(Sender: TObject);
begin
 SetRmtViewBtn(8);
end;

procedure TiXplorerForm.MenuItemRmtDetailsClick(Sender: TObject);
begin
 SetRmtViewBtn(9);
end;

function TiXplorerForm.PasswordFound(const id: String;
                                     var retPwd: TiXplorerPwd): Boolean;
var
 i: Integer;
 maxInx: Integer;
 found: Boolean;
begin
 i:=0;
 maxInx:=pwdList.Count;
 found:=False;
 while (i < maxInx) and not found do
  begin
   retPwd:=pwdList.Items[i] as TiXplorerPwd;
   found:=(pwdItem.GetID = id);
   Inc(i);
  end;
 PasswordFound:=found;
end;

function TiXplorerForm.PasswordStringFound(const id: String;
                                     var retStr: String): Boolean;
var
 dummyPwd: TiXplorerPwd;
 found: Boolean;
begin
 retStr:='';
 found:=PasswordFound(id,dummyPwd);
 if found then
   retStr:=pwdItem.GetPassword;
 PasswordStringFound:=found;
end;

function TiXplorerForm.GetPassword(const id: String; var ok, usedDialog: Boolean): String;
var
 loginForm: TLoginForm;
begin
 usedDialog:=False;
 ok:=False;
 GetPassword:='';
 if not PasswordFound(id,pwdItem) then
  begin
   pwdItem:=TiXplorerPwd.Create(id);
   pwdList.Add(pwdItem);
  end;
 if pwdItem.PasswordIsOK then
  begin
   ok:= True;
   GetPassword:=pwdItem.GetPassword;
  end
 else
  begin
   loginForm:=TLoginForm.Create(Self);
   loginForm.NetworkIdEdit.Text:=
     fIniFile.ReadString(id, 'Username', 'guest') +'@'
        +fIniFile.ReadString(id, 'Host', '');
   pwdEntered:=True;
   if (loginForm.ShowModal = mrOK) then
    begin
     ok:= True;
     usedDialog:=True;
     GetPassword:=loginForm.PwdEdit.Text;
     pwdItem.SetPassword(loginForm.PwdEdit.Text);
    end
   else
     GetPassword:='';
   loginForm.Free;
  end;
end;

procedure TiXplorerForm.WriteRegistry;
begin
 fIniFile.WriteString(sectionGeneral, 'HostList', hostList.CommaText);
 fIniFile.WriteBool(sectionGeneral, 'Keep', KeepMenu.Checked);
end;

procedure TiXplorerForm.ReadRegistry;
begin
 hostList.CommaText := fIniFile.ReadString(sectionGeneral, 'HostList', '');
 KeepMenu.Checked := fIniFile.ReadBool(sectionGeneral, 'Keep', True);
end;

procedure TiXplorerForm.HostWriteRegistry(const idStr: String;
                                          const itemRec: TiXplorerHostEntry);
begin
 fIniFile.WriteBool(idStr, 'DefaultUsername', itemRec.UseDefaultUsername);
 fIniFile.WriteBool(idStr, 'DefaultPassword', itemRec.UseDefaultPwd);
 fIniFile.WriteString(idStr, 'Username', itemRec.GetUsername);
 fIniFile.WriteString(idStr, 'Host', itemRec.GetHostPath);
 fIniFile.WriteString(idStr, 'LocalDir', itemRec.GetLocalDirPath);
 fIniFile.WriteString(idStr, 'PortNumber', itemRec.GetPortNumStr);
 fIniFile.WriteInteger(idStr, 'Timeout', itemRec.GetTimeout);
 fIniFile.WriteInteger(idStr, 'FilePosition', itemRec.GetFilepos);
 fIniFile.WriteBool(idStr, 'PreserveFileAttributes', itemRec.PreserveAttribIsSet);
 fIniFile.WriteBool(idStr, 'HideConsole', itemRec.HideConsoleIsSet);
 fIniFile.WriteBool(idStr, 'FixFileposition', itemRec.FixPositionIsSet);
end;

procedure TiXplorerForm.GetHostEntry(const id: String; var ret: TiXplorerHostEntry);
begin
 ret.SetDefaultUsername(fIniFile.ReadBool(id, 'DefaultUsername', False));
 ret.SetDefaultPwd(fIniFile.ReadBool(id, 'DefaultPassword', False));
 ret.SetUsername(fIniFile.ReadString(id, 'Username', 'guest'));
 ret.SetHostPath(fIniFile.ReadString(id, 'Host', ''));
 ret.SetLocalDirPath(fIniFile.ReadString(id, 'LocalDir', ''));
 ret.SetPortNumStr(fIniFile.ReadString(id, 'PortNumber', ''));
 ret.SetTimeout(fIniFile.ReadInteger(id, 'Timeout', defaultTimeout));
 ret.SetFilepos(fIniFile.ReadInteger(id, 'FilePosition', defaultFilepos));
 ret.SetPreserveAttrib(fIniFile.ReadBool(id, 'PreserveFileAttributes', False));
 ret.SetHideConsole(fIniFile.ReadBool(id, 'HideConsole', True));
 ret.SetFixPosition(fIniFile.ReadBool(id, 'FixFileposition', False));
end;

procedure TiXplorerForm.RmtAddHostToolBtnClick(Sender: TObject);
var
 setupForm: TSetupForm;
 i: Integer;
 hostEntry: TiXplorerHostEntry;
begin
 setupForm := TSetupForm.Create(Self);
 with setupForm do
  begin
   CheckBoxHideConsole.Checked:=True; // This is the default value
   if (ShowModal() = mrOk)
     and (Length(NameEdit.Text)>0)
     and ((Length(UsernameEdit.Text)>0) or (setupForm.UsernameCheckBox.Checked))
     and (Length(HostEdit.Text)>0) then
    begin
     if not hostList.Find(NameEdit.Text,i) then // is Unique ID ?
      begin
       hostEntry:=TiXplorerHostEntry.Create;
       with hostEntry do
        begin
         SetDefaultUsername(UsernameCheckBox.Checked);
         SetDefaultPwd(DefaultPasswordCheckBox.Checked);
         SetUsername(UsernameEdit.Text);
         SetHostPath(HostEdit.Text);
         SetLocalDirPath(LocalDirEdit.Text);
         SetPortNumStr(PortEdit.Text);
         SetTimeout(SpinEditTimeout.Value);
         SetFilepos(SpinEditFilePos.Value);
         SetPreserveAttrib(CheckBoxPreserveAttributes.Checked);
         CheckBoxHideConsole.Checked:=HideConsoleIsSet;
         CheckBoxFixPos.Checked:=FixPositionIsSet;
        end;
       HostWriteRegistry(NameEdit.Text,hostEntry);
       hostList.Add(NameEdit.Text);
       hostList.Sort; // Must be sorted for the Find command to work
       WriteRegistry;
       hostEntry.Free;
       Load(nil);
      end;
//   else Error Message: Already exists
    end;
  end;
 setupForm.Free;
end;

procedure TiXplorerForm.RmtDelHostToolBtnClick(Sender: TObject);
var
 i: Integer;
begin
 MenuItemOpenNodeClick(Sender);
 if (selNode = nil) or (selNode.Level < 1) then Exit;
 if (Application.MessageBox(PChar(
                'Are you sure you want to remove the connection to '
              + parentNode.Text
              + ' ?'),
                'Remove SSH Connection', MB_YESNO) = IDYES)
   and (hostList.Find(parentNode.Text,i)) then
    begin
     hostList.Delete(i);
     WriteRegistry;
     Load(nil);
    end;
end;

procedure TiXplorerForm.MenuItemTreeViewPropertyClick(Sender: TObject);
var
 setupForm: TSetupForm;
 i: Integer;
 hostEntry: TiXplorerHostEntry;
 locPwdStr: String;
begin
 MenuItemOpenNodeClick(Sender);
 if (selNode = nil) or (selNode.Level < 1) then Exit;
 setupForm := TSetupForm.Create(Self);
 with setupForm do
  begin
   hostEntry:=TiXplorerHostEntry.Create;
   GetHostEntry(parentNode.Text,hostEntry);
   with hostEntry do
    begin
     UsernameCheckBox.Checked:=UseDefaultUsername;
     DefaultPasswordCheckBox.Checked:=UseDefaultPwd;
     UsernameEdit.Text:=GetUsername;
     HostEdit.Text:=GetHostPath;
     LocalDirEdit.Text:=GetLocalDirPath;
     PortEdit.Text:=GetPortNumStr;
     NameEdit.Text:=selNodeStr;
     SpinEditTimeout.Value:=GetTimeout;
     SpinEditFilePos.Value:=GetFilepos;
     CheckBoxPreserveAttributes.Checked:=PreserveAttribIsSet;
     CheckBoxHideConsole.Checked:=HideConsoleIsSet;
     CheckBoxFixPos.Checked:=FixPositionIsSet;
    end;
   NameEdit.ReadOnly:=True;
   NameEdit.Color:=clMenu;
   if PasswordStringFound(NameEdit.Text,locPwdStr) then
     SetPassword(locPwdStr);
   if (ShowModal() = mrOk)
     and ((Length(UsernameEdit.Text)>0) or (setupForm.UsernameCheckBox.Checked))
     and (Length(HostEdit.Text)>0) then
    begin
     if hostList.Find(NameEdit.Text,i) then
       hostList.Delete(i);
     with hostEntry do
      begin
       SetDefaultUsername(UsernameCheckBox.Checked);
       SetDefaultPwd(DefaultPasswordCheckBox.Checked);
       SetUsername(UsernameEdit.Text);
       SetHostPath(HostEdit.Text);
       SetLocalDirPath(LocalDirEdit.Text);
       SetPortNumStr(PortEdit.Text);
       SetTimeout(SpinEditTimeout.Value);
       SetFilepos(SpinEditFilePos.Value);
       SetPreserveAttrib(CheckBoxPreserveAttributes.Checked);
       SetHideConsole(CheckBoxHideConsole.Checked);
       SetFixPosition(CheckBoxFixPos.Checked);
      end;
     HostWriteRegistry(NameEdit.Text,hostEntry);
     hostList.Add(NameEdit.Text);
     hostList.Sort; // Must be sorted for the Find command to work
     WriteRegistry;
     Load(nil);
    end;
   NameEdit.ReadOnly:=False;
   NameEdit.Color:=clWindow;
   hostEntry.Free;
  end;
 setupForm.Free;
end;

procedure TiXplorerForm.MenuItemOpenNodeClick(Sender: TObject);
begin
 if (mouseCurNode <> nil) then
  begin
   DirTreeView.Selected:=mouseCurNode;
   DirTreeViewChange(Self,DirTreeView.Selected);
  end;
end;

procedure TiXplorerForm.NoResponseTimerTimer(Sender: TObject);
begin
 if (noResponseCnt = 1) then
  begin
   Screen.Cursor := crDefault;
   pwdEntered:=False;
   bLoading:=False;
   noResponseCnt:=0;
   Application.MessageBox('SSH Host Timeout','No response from host',MB_OK);
  end
 else if (noResponseCnt > 1) then
   Dec(noResponseCnt);
end;

procedure TiXplorerForm.RmtUpToolBtnClick(Sender: TObject);
begin
 if (selNode <> nil) then
  begin
   DirTreeView.Selected:=selNode.Parent;
   DirTreeViewChange(Sender,DirTreeView.Selected);
  end;
end;

procedure TiXplorerForm.LocUpToolBtnClick(Sender: TObject);
begin
 systemTreeView.ChangeToParent;
end;

procedure TiXplorerForm.LocAddFolderToolBtnClick(Sender: TObject);
var
 folderForm: TNewFolderForm;
begin
 folderForm:=TNewFolderForm.Create(Self);
 folderForm.LocationEdit.Text:=systemTreeView.Directory;
 if (folderForm.ShowModal = mrOK)
  and (Length(folderForm.NameEdit.Text) > 0) then
   systemTreeView.AddNewNode(systemTreeView.Selected, folderForm.NameEdit.Text, True);
 folderForm.Free;
end;

procedure TiXplorerForm.MenuItemOpenClick(Sender: TObject);
begin
  Show;
  ShowWindow(Application.Handle, SW_HIDE);
end;

procedure TiXplorerForm.HelpMenuClick(Sender: TObject);
var
 str: String;
begin
 str :=
'No help available yet!' +
#13#10 +
#13#10 +
'Some information and help can be found at: ' +
'http://i-tree.org';
  Application.MessageBox(PChar(str),'Not yet implemented',MB_OK);
end;

procedure TiXplorerForm.OpenObject(sObjectPath : PChar);
begin
 ShellExecute(0, Nil, sObjectPath, Nil, Nil, SW_NORMAL);
end;

procedure TiXplorerForm.HomePageMenuClick(Sender: TObject);
var
 str: String;
begin
 str:='http:\\i-tree.org';
 OpenObject(PChar(str));
end;

procedure TiXplorerForm.RmtCopyToolBtnClick(Sender: TObject);
var
 i: Integer;
 prefixStr: String;
begin
 if Length(selDirStr) > 0 then
   prefixStr:=sshPrefix+selHostStr+selDirStr
 else
   prefixStr:=sshPrefix+selHostStr;
 DropFileSource1.Files.clear;
 for i := 0 to FileListView.items.Count-1 do
   if (FileListView.items[i].Selected)
   // To Do!!! Implement handling links -> presently they are disabled here
    and (FileListView.items[i].ImageIndex <> 4) then
      DropFileSource1.Files.Add(prefixStr+FileListView.items.item[i].caption);
 DropFileSource1.CopyToClipboard;
 RmtPasteToolBtn.Enabled:=False;
 LocPasteToolBtn.Enabled:=DropFileSource1.Files.Count > 0;
 MenuItemLocPaste.Enabled:=LocPasteToolBtn.Enabled;
 MenuItemRmtPaste.Enabled:=RmtPasteToolBtn.Enabled;
end;

procedure TiXplorerForm.LocCopyToolBtnClick(Sender: TObject);
var
 i: Integer;
begin
 DropFileSource2.Files.clear;
 //Fill DropSource1.Files with selected filenames in ListView1...
 for i := 0 to SystemListView.items.Count-1 do
   if (SystemListView.items.item[i].Selected) then
      DropFileSource2.Files.Add(
        SystemListView.FullFilename[SystemListView.items.item[i]]);
 DropFileSource2.CopyToClipboard;
 LocPasteToolBtn.Enabled:=False;
 RmtPasteToolBtn.Enabled:=DropFileSource2.Files.Count > 0;
 MenuItemLocPaste.Enabled:=LocPasteToolBtn.Enabled;
 MenuItemRmtPaste.Enabled:=RmtPasteToolBtn.Enabled;
end;

procedure TiXplorerForm.RmtPasteToolBtnClick(Sender: TObject);
var
 i: integer;
 ListItem: TListItem;
 fNameStr: String;
begin
 if (Length(selNodeStr) > 0) then
  begin
   DropFileTarget1.PasteFromClipboard;
   with DropFileTarget1 do
    for i := 0 to files.count-1 do
     begin
      fNameStr:=files[i];
      if (Pos(sshPrefix,fNameStr) <> 1) then
       begin
        lockListAccess.Acquire; // Lock out other threads
        ListItem := CopyListView.Items.Add;
        ListItem.Caption := fNameStr;
        ListItem.SubItems.Add(sshPrefix+selHostStr+selDirStr);
        ListItem.SubItems.Add(selNodeStr);
        lockListAccess.Release;
       end;
     end;
  end;
 LocPasteToolBtn.Enabled:=False;
 RmtPasteToolBtn.Enabled:=False;
 MenuItemLocPaste.Enabled:=LocPasteToolBtn.Enabled;
 MenuItemRmtPaste.Enabled:=RmtPasteToolBtn.Enabled;
end;

procedure TiXplorerForm.LocPasteToolBtnClick(Sender: TObject);
var
 i: integer;
 ListItem: TListItem;
 targetStr,fNameStr: String;
 nonSshFileFound: Boolean;
begin
 nonSshFileFound:=False;
 if (Length(selNodeStr) > 0) then
  begin
   DropFileTarget2.PasteFromClipboard;
   with DropFileTarget2 do
    for i := 0 to files.count-1 do
     begin
      fNameStr:=files[i];
      if (Pos(sshPrefix,fNameStr) <> 1) then
        nonSshFileFound:=True
      else
       begin
        lockListAccess.Acquire; // Lock out other threads
        ListItem := CopyListView.Items.Add;
        ListItem.Caption := fNameStr;
        targetStr:=IncludeTrailingBackslash(SystemTreeView.Directory);
// New in Version 0.9
        if (Length(targetStr)=3) and (targetStr[2] = ':') then
          //This is a disk letter spec and must be without any '\' character
          targetStr:=ExcludeTrailingBackslash(SystemTreeView.Directory);
        ListItem.SubItems.Add(targetStr);
        ListItem.SubItems.Add(selNodeStr);
        lockListAccess.Release;
       end;
     end;
   LocPasteToolBtn.Enabled:=False;
   RmtPasteToolBtn.Enabled:=False;
   MenuItemLocPaste.Enabled:=LocPasteToolBtn.Enabled;
   MenuItemRmtPaste.Enabled:=RmtPasteToolBtn.Enabled;
  end
 else
   Application.MessageBox(noSshSelectionErrStr,errStr3,MB_OK);
 if nonSshFileFound then
   Application.MessageBox(nonSshFileErrStr,errStr2,MB_OK);
end;

procedure TiXplorerForm.ClearQueueMenuItemClick(Sender: TObject);
begin
 CancelCopying;
end;

// New in Version 0.8
procedure TiXplorerForm.FileViewKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
 if (Key = 116) then //F5
   SelectDirTreeNode(selNode,True)
 else if (Key = 112) then //F1
   HomePageMenuClick(Sender);
end;

// New in Version 0.8
procedure TiXplorerForm.SystemViewKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
 if (Key = 116) then //F5
  begin
   SystemTreeView.Reset;
   SystemListView.Reset;
  end
 else if (Key = 112) then //F1
   HomePageMenuClick(Sender);
end;

// New in Version 0.9
procedure TiXplorerForm.DownloadToolButtonClick(Sender: TObject);
begin
 RmtCopyToolBtnClick(Sender);
 LocPasteToolBtnClick(Sender);
end;

// New in Version 0.9
procedure TiXplorerForm.UploadToolButtonClick(Sender: TObject);
begin
 LocCopyToolBtnClick(Sender);
 RmtPasteToolBtnClick(Sender);
end;

end.

